@@ -1,11 +1,12 @@
 /*
- * $Id: p_header,v 1.2 2011-02-23 17:47:10 koshiro Exp $
+ * $Id: p_header,v 1.3 2015/03/18 03:27:42 horinout Exp $
  */
 
 #include <stdio.h>
 #include "ruby.h"
 #include "libtinyf2c.h"
 #include "narray.h"
+#include "cdcl.h"
 
 /* for compatibility with ruby 1.6 */
 #ifndef StringValuePtr
@@ -409,13 +410,21 @@ dcl_swgton(obj, np, wpx, wpy, itpat)
 }
 
 static VALUE
-dcl_swiopn(obj, iwx, iwy, imw, imh)
-    VALUE obj, iwx, iwy, imw, imh;
+dcl_swiopn(obj, iwx, iwy, imw, imh, wx1, wy1, wx2, wy2, wx3, wy3, wx4, wy4)
+    VALUE obj, iwx, iwy, imw, imh, wx1, wy1, wx2, wy2, wx3, wy3, wx4, wy4;
 {
     integer i_iwx;
     integer i_iwy;
     integer i_imw;
     integer i_imh;
+    real i_wx1;
+    real i_wy1;
+    real i_wx2;
+    real i_wy2;
+    real i_wx3;
+    real i_wy3;
+    real i_wx4;
+    real i_wy4;
 
     if ((TYPE(iwx) != T_BIGNUM) || (TYPE(iwx) != T_FIXNUM)) {
       iwx = rb_funcall(iwx, rb_intern("to_i"), 0);
@@ -429,14 +438,46 @@ dcl_swiopn(obj, iwx, iwy, imw, imh)
     if ((TYPE(imh) != T_BIGNUM) || (TYPE(imh) != T_FIXNUM)) {
       imh = rb_funcall(imh, rb_intern("to_i"), 0);
     }
+    if (TYPE(wx1) != T_FLOAT) {
+      wx1 = rb_funcall(wx1, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wy1) != T_FLOAT) {
+      wy1 = rb_funcall(wy1, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wx2) != T_FLOAT) {
+      wx2 = rb_funcall(wx2, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wy2) != T_FLOAT) {
+      wy2 = rb_funcall(wy2, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wx3) != T_FLOAT) {
+      wx3 = rb_funcall(wx3, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wy3) != T_FLOAT) {
+      wy3 = rb_funcall(wy3, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wx4) != T_FLOAT) {
+      wx4 = rb_funcall(wx4, rb_intern("to_f"), 0);
+    }
+    if (TYPE(wy4) != T_FLOAT) {
+      wy4 = rb_funcall(wy4, rb_intern("to_f"), 0);
+    }
 
     i_iwx = NUM2INT(iwx);
     i_iwy = NUM2INT(iwy);
     i_imw = NUM2INT(imw);
     i_imh = NUM2INT(imh);
+    i_wx1 = (real)NUM2DBL(wx1);
+    i_wy1 = (real)NUM2DBL(wy1);
+    i_wx2 = (real)NUM2DBL(wx2);
+    i_wy2 = (real)NUM2DBL(wy2);
+    i_wx3 = (real)NUM2DBL(wx3);
+    i_wy3 = (real)NUM2DBL(wy3);
+    i_wx4 = (real)NUM2DBL(wx4);
+    i_wy4 = (real)NUM2DBL(wy4);
 
 
-    swiopn_(&i_iwx, &i_iwy, &i_imw, &i_imh);
+    swiopn_(&i_iwx, &i_iwy, &i_imw, &i_imh, &i_wx1, &i_wy1, &i_wx2, &i_wy2, &i_wx3, &i_wy3, &i_wx4, &i_wy4);
 
     return Qnil;
 
@@ -2021,7 +2062,7 @@ VALUE mDCL;
     rb_define_module_function(mDCL, "swstcl", dcl_swstcl, 1);
 #endif
     rb_define_module_function(mDCL, "swgton", dcl_swgton, 4);
-    rb_define_module_function(mDCL, "swiopn", dcl_swiopn, 4);
+    rb_define_module_function(mDCL, "swiopn", dcl_swiopn, 12);
     rb_define_module_function(mDCL, "swidat", dcl_swidat, 2);
 #if DCLVER >= 544
     rb_define_module_function(mDCL, "swiclr", dcl_swiclr, 2);
